/*
 * MainFrame: メインウィンドウ
 * 注記: BridgeパターンのAbstractionクラス
 */

package org.dbc_works.app.sbkeyhook;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.dbc_works.lib.environment.EnvInfo;
import org.dbc_works.lib.model.Command;
import org.dbc_works.lib.model.CommandList;
import org.dbc_works.lib.ArgumentInfo;
import org.dbc_works.lib.ui.ActionManager;
import org.dbc_works.lib.ui.MainFrameBase;

import org.dbc_works.app.sbkeyhook.cmd.AddAccountInfoCommand;
import org.dbc_works.app.sbkeyhook.cmd.DeleteAccountInfoCommand;
import org.dbc_works.app.sbkeyhook.cmd.EditAccountInfoCommand;
import org.dbc_works.app.sbkeyhook.cmd.UpdateAccountInfoCommand;
import org.dbc_works.app.sbkeyhook.model.AccountInfo;
import org.dbc_works.app.sbkeyhook.model.KeyHookDocument;
import org.dbc_works.app.sbkeyhook.model.Serializer;
import org.dbc_works.app.sbkeyhook.view.LabelManager;
import org.dbc_works.app.sbkeyhook.view.ImplementOperation;

public final class MainFrame extends MainFrameBase
{
	//
	// 定数
	//

	private static final String DIALOG_PASSWORD = "PasswordDialog";

	//
	// クラスフィールド(実装)
	//

	private static boolean DEBUG = false;

	private static final MainFrame instance = new MainFrame();

	//
	// クラスメソッド(公開)
	//

	// 唯一のインスタンスを返す
	public static MainFrame getInstance()
	{
		return instance;
	}

	//
	// フィールド(公開)
	//
	
	// 追加アクション
	final ActionListener addAction	= new ActionListener() {
		// アクションを実行する
		public void actionPerformed(
			ActionEvent event)	// アクションイベント
		{
			EventQueue.invokeLater(new Runnable() {
				public void run() {
					editAccountInfo(true);
				}
			});
		}
	};
	
	// 編集アクション
	final ActionListener editAction	= new ActionListener() {
		// アクションを実行する
		public void actionPerformed(
			ActionEvent event)	// アクションイベント
		{
			EventQueue.invokeLater(new Runnable() {
				public void run() {
					editAccountInfo(false);
				}
			});
		}
	};
	
	// 削除アクション
	final ActionListener deleteAction	= new ActionListener() {
		// アクションを実行する
		public void actionPerformed(
			ActionEvent event)	// アクションイベント
		{
			EventQueue.invokeLater(new Runnable() {
				public void run() {
					deleteAccountInfo();
				}
			});
		}
	};
	
	// 取り消しアクション
	final ActionListener undoAction	= new ActionListener() {
		// アクションを実行する
		public void actionPerformed(
			ActionEvent event)	// アクションイベント
		{
			final Properties label = LabelManager.getLabelProperties();
			try {
				if (commands.canUndo()) {
					setWaitCursor();
					final String commandName = commands.getUndoCommandName(); 
					final Command.Result result = commands.undo();
	
					if (result == Command.Result.SUCCESS) {
						updateView(true, "");
						setStatusMessage(label.getProperty("Status_Undo") + commandName);
					}
				}
				else {
					showMessage(label.getProperty("Msg_Info_CannotUndo"));
				}
			}
			catch (Exception ex) {
				outputExceptionInfo(ex);
				showMessage(ex.toString());
				setStatusMessage("");
				commands.setFixed();
			}
			finally {
				setActionStatus();
				restoreCursor();
			}
		}
	};

	// やりなおしアクション
	final ActionListener redoAction	= new ActionListener() {
		// アクションを実行する
		public void actionPerformed(
			ActionEvent event)	// アクションイベント
		{
			try {
				if (commands.canRedo()) {
					setWaitCursor();
					final String commandName = commands.getRedoCommandName(); 
					final Command.Result result = commands.redo();
	
					if (result == Command.Result.SUCCESS) {
						updateView(true, "");
						setStatusMessage(LabelManager.getLabelProperties().getProperty("Status_Redo") + commandName);
					}
				}
				else {
					showMessage(LabelManager.getLabelProperties().getProperty("Msg_Info_CannotRedo"));
				}
			}
			catch (Exception ex) {
				outputExceptionInfo(ex);
				showMessage(ex.toString());
				setStatusMessage("");
				commands.removeRedoCommand();
			}
			finally {
				setActionStatus();
				restoreCursor();
			}
		}
	};

	//
	// フィールド(実装)
	//

	private KeyHookDocument document = new KeyHookDocument();
	private final CommandList commands = new CommandList();
	private ActionManager actionManager;
	private String documentPath;

	//
	// コンストラクタ
	//
	private MainFrame()
	{
		super(true);
	}

	//
	// 操作
	//

	// アクションの状態を設定する
	public void setActionStatus()
	{
		actionManager.setEnable(undoAction, commands.canUndo());
		actionManager.setEnable(redoAction, commands.canRedo());
		
		final ImplementOperation op = getOperation();
		if (op != null) {
			actionManager.setEnable(editAction, op.hasSelection());
			actionManager.setEnable(deleteAction, op.hasSelection());
		}
	}

	//
	// 実装
	//

	// フレームをImplementOperationインターフェースとして返す
	private ImplementOperation getOperation()
	{
		return (ImplementOperation)getImplementor();
	}

	// ドキュメントのパスを返す
	private String getDocumentPath()
	{
		String path = (documentPath != null ? documentPath : EnvInfo.getInstance().getUserHomeDir());
		if (path.endsWith(File.separator) == false) {
			path += File.separator;
		}
		return  path;
	}

	// ドキュメントのファイルを返す
	private File getDocumentFile()
	{
		return new File(getDocumentPath() + "KeyHook.dat");
	}

	// 実装を初期化する
	private void initImplementor(
		Frame frame)	// 実装(フレーム)
	{
		if (DEBUG) {
			if (frame == null) {
				throw new NullPointerException();
			}
		}
		
		final EnvInfo info = EnvInfo.getInstance();
		frame.setTitle(getAppName(false));

		// フォントを設定する
		frame.setFont(info.getBaseFont(frame));

		// メニューとツールバーを作成する
		final ControllerDefinition definition = new ControllerDefinition();
		final Component[] tools = definition.buildMainController(this, frame, actionManager);

		final ImplementOperation op = (ImplementOperation)frame;
		op.initComponent();
		op.addToolBarComponents(tools);
		op.setListSelectionReceiver(new AbstractionReceiver() {
			public void receive()
			{
				updateView(false, op.hasSelection() ? op.getSelectedAccountName() : "");
				setActionStatus();
			}
		});
		op.setEditAccountReceiver(new AbstractionReceiver() {
			public void receive()
			{
				editAccountInfo(false);
			}
		});

		setActionStatus();
	}
	
	// ドキュメントを初期化する
	private boolean initDocument(
		File docFile)	// ドキュメントファイルパス
	{
		if (DEBUG) {
			if (docFile == null) {
				throw new NullPointerException();
			}
		}

		boolean success = false;

		final ImplementOperation op = getOperation(); 
		try {
			while (success == false) {
				final String password = op.showPasswordDialog(getFrameTitle(DIALOG_PASSWORD), false);
				if (password == null) {
					break;
				}

				document = Serializer.getDeserializeInstance(password).deserialize(docFile);
				if (document != null) {
					updateView(true, "");
					setActionStatus();
					success = true;
				}
			}
		}
		catch (Exception ex) {
			outputExceptionInfo(ex);
			showMessage(ex.toString());
			success = op.showConfirmMessage(LabelManager.getLabelProperties().getProperty("Msg_Confirm_Start"));
		}
		return success;
	}
	
	// 表示を更新する
	private void updateView(
		boolean updateList,	// リスト更新フラグ
		String selectName)		// 選択アカウント情報名
	{
		final ImplementOperation op = getOperation();
		if (updateList) {
			op.setAccountInfoNames(document.getNames());
		}
		final AccountInfo info = (0 < selectName.length()
								? document.getAccountInfoOf(selectName)
								: null);  
		op.selectAccount(info);
	}
	
	// 状態メッセージを設定する
	private void setStatusMessage(
		String msg)	// 設定メッセージ
	{
		if (DEBUG) {
			if (msg == null) {
				throw new NullPointerException();
			}
		}

		getOperation().setStatusMessage(0 < msg.length() ? msg : LabelManager.getLabelProperties().getProperty(LabelManager.STATUS_READY));
	}
	
	// コマンドを実行する
	private boolean executeCommand(
		EditAccountInfoCommand command)	// 実行コマンド
		throws Exception
	{
		if (DEBUG) {
			if (command == null) {
				throw new NullPointerException();
			}
		}

		Command.Result result;
		try {	
			result = commands.execute(command);
		
			if (result == Command.Result.SUCCESS) {
				updateView(command.isEditCommand() == false, command.getAddAccountInfoName());
				setStatusMessage(LabelManager.getLabelProperties().getProperty("Status_Success"));
			}
			else {
				setStatusMessage("");
			}
		}
		finally {
			setActionStatus();
		}
		
		return (result == Command.Result.SUCCESS);
	}
	
	// アカウント情報を編集する
	private void editAccountInfo(
		boolean add)	// 追加情報
	{
		final ImplementOperation op = getOperation();
		final Properties label = LabelManager.getLabelProperties();
		 
		try {
			// 編集ダイアログを表示する
			EditAccountInfoCommand cmd = null;
			AccountInfo account;
			if (add) {
				account = op.showAddAccountInfoDialog(getFrameTitle("AddAccountInfoDialog"), document.getNames());
				if (account != null) {
					cmd = new AddAccountInfoCommand(account, document);
				}
			}
			else {
				if (op.hasSelection() == false) {
					op.showMessage(label.getProperty(LabelManager.MSG_NOSELECTION));
					return;
				}
				final AccountInfo currentAccount = document.getAccountInfoOf(op.getSelectedAccountName());
				account = op.showEditAccountInfoDialog(getFrameTitle("EditAccountInfoDialog"), currentAccount);
				if (account != null) {
					cmd = new UpdateAccountInfoCommand(currentAccount, account, document);
				}
			}
			if (cmd == null) {
				return;
			}

			setWaitCursor();
			executeCommand(cmd);
		}
		catch (Exception ex) {
			outputExceptionInfo(ex);
			showMessage(ex.toString());
			setStatusMessage("");
		}
		finally {
			restoreCursor();
		}
	}
	
	// アカウント情報を削除する
	private void deleteAccountInfo()
	{
		final ImplementOperation op = getOperation();
		final Properties label = LabelManager.getLabelProperties(); 

		if (op.hasSelection() == false) {
			showMessage(label.getProperty(LabelManager.MSG_NOSELECTION));
			return;
		}

		try {
			if (op.showConfirmMessage(label.getProperty("Msg_Confirm_Delete")) == false) {
				return;
			}
			
			setWaitCursor();
			executeCommand(new DeleteAccountInfoCommand(document.getAccountInfoOf(op.getSelectedAccountName()), document));
		}
		catch (Exception ex) {
			outputExceptionInfo(ex);
			showMessage(ex.toString());
			setStatusMessage("");
		}
		finally {
			restoreCursor();
		}
	}
	
	//
	// 再定義メソッド(実装)
	//

	// アプリケーション名を返す	
	protected String getAppName(
		boolean bShort)	// 短縮名フラグ
	{
		return (bShort != false ? "sbkeyhook" : "StompBox/KeyHook");
	}

	// 実装を作成して返す
	// 注記: J2MEに存在しないクラスのロードを避けるためにリフレクションで作成する
	protected Frame createImplementor(
		ArgumentInfo argument)	// 引数情報
	{
		if (DEBUG) {
			if (argument == null) {
				throw new NullPointerException();
			} 
		}

		Frame frame = null;

		try {
			final boolean asAwt = isToolkitAwt(argument);
			if (asAwt == false) {
				initSwing(argument);
			//	frame = new swing.MainFrameImp();
				frame = (Frame)Class.forName("org.dbc_works.app.sbkeyhook.view.swing.MainFrameImp").newInstance();
			}
			else {
				frame = new org.dbc_works.app.sbkeyhook.view.awt.MainFrameImp();
			//	frame = (Frame)Class.forName("org.dbc_works.app.sbfiles.view.awt.MainFrameImp").newInstance();
			}
			actionManager = ActionManager.createInstance(asAwt);

			initImplementor(frame);
		}
		catch (ClassNotFoundException ex) {
			// Do nothing.
			outputExceptionInfo(ex);
		}
		catch (IllegalAccessException ex) {
			// Do nothing.
			outputExceptionInfo(ex);
		}
		catch (InstantiationException ex) {
			// Do nothing.
			outputExceptionInfo(ex);
		}

		return frame;		
	}

	// デフォルトのサイズを返す
	protected Dimension getDefaultSize()
	{
		final Frame frame = getImplementor();
		if (DEBUG) {
			if (frame == null) {
				throw new IllegalStateException();
			}
		}

		final Dimension screenSize = frame.getToolkit().getScreenSize();
		return new Dimension(screenSize.width / 4, screenSize.height / 2);
	}

	// 初期化する
	protected boolean doInit(
		ArgumentInfo argument)	// 引数情報
	{
		setTitle(getAppName(false) + " Ver.0.1");
		setIcon();

		final String OPTION_HOOKPATH = "HookPath"; 
		if (argument.hasOption(OPTION_HOOKPATH)) {
			String path = argument.getOptionValue(OPTION_HOOKPATH);
			if (path.equals("beside")) {
				path = System.getProperty("user.dir");
			}
			if (new File(path).exists()) {
				documentPath = path; 
			}
		}
		setStatusMessage("");
		return true;
	}

	// 追加状態を復元する
	protected void restoreAdditionalState()
	{
		try {
			getOperation().setAdditionalState(getStateProperties());
		}
		catch (Exception ex) {
			// Do nothing.
			outputExceptionInfo(ex);
		}
	}

	// 追加状態を設定する
	protected void setAdditionalStatePropreties()
	{
		final Map mapAdditionalState = getOperation().getAdditionalState();
		final Set keySet = mapAdditionalState.keySet();
		final Iterator it = keySet.iterator();
		while (it.hasNext()) {
			String strKey = (String)it.next();
			setStatePropertyValue(strKey, (String)mapAdditionalState.get(strKey));
		}
	}

	// 終了可能かを返す
	protected synchronized boolean canExit()
	{
		boolean canClose = true;
		if (commands.canUndo() && document.isModified()) {
			final ImplementOperation op = getOperation();
			try {
				final String password = op.showPasswordDialog(getFrameTitle(DIALOG_PASSWORD), true);
				if (password == null) {
					canClose = op.showConfirmMessage(LabelManager.getLabelProperties().getProperty("Msg_Confirm_Close"));
				}
				if (password != null) {
					Serializer.getSerializeInstance(password, document).serialize(getDocumentFile(), new File(getDocumentPath() + "KeyHook.bak"));
				}
			}
			catch (Exception ex) {
				outputExceptionInfo(ex);
				showMessage(ex.toString());
				canClose = op.showConfirmMessage(LabelManager.getLabelProperties().getProperty("Msg_Confirm_ContinueClose"));
			}
		}
		return canClose;
	}

	// 終了処理を行う
	protected void prepareExit()
	{
		// 何もしない
	}
	
	//
	// 再定義メソッド(WindowAdapter)
	//

	// ウィンドウ作成終了
	public void windowOpened(
		WindowEvent e)	// イベント情報
	{
		boolean initialized = true;
		final File file = getDocumentFile();
		if (file.exists()) {
			initialized = initDocument(file);
		}
		
		if (initialized) {
			setActionStatus();			
		}
		else {
			final Frame frame = getImplementor(); 
			frame.getToolkit().getSystemEventQueue().postEvent(new WindowEvent(frame, WindowEvent.WINDOW_CLOSING));
		}
	}
}
