/*
 * KeyHookDocument: ドキュメント
 */
package org.dbc_works.app.sbkeyhook.model;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Iterator;

import org.dbc_works.app.sbkeyhook.SBKeyHook;

public final class KeyHookDocument
{
	//
	// 定数
	//
	
	private static final String CHARSET = "UTF8";
	private static final char NEWLINE = '\n';
	
	//
	// クラスメソッド(公開)
	//

	// 入力を解析する
	static KeyHookDocument parse(
		InputStream stream)	// 入力対象ストリーム
		throws IOException
	{
		final Reader reader = new BufferedReader(new InputStreamReader(stream, CHARSET));
		final List accounts = new ArrayList();
		try {
			final StringBuffer sb = new StringBuffer(); 
			for (int c = reader.read(); 0 <= c; c = reader.read()) {
				if (c == NEWLINE) {
					accounts.add(AccountInfo.parse(sb.toString()));
					sb.setLength(0);
				}
				else {
					sb.append((char)c);
				}
			}
		}
		finally {
			try {
				reader.close();
			}
			catch (Exception ex){
				// Do nothing
				SBKeyHook.reportException(ex);
			}
		}
		return new KeyHookDocument(accounts);
	}

	//
	// フィールド(実装)
	//

	private final List accounts = new ArrayList();
	private boolean modified;

	//
	// コンストラクタ
	//
	public KeyHookDocument()
	{
	}
	private KeyHookDocument(
		List a)	// アカウント情報の集合
	{
		if (a == null) {
			throw new NullPointerException();
		}

		accounts.addAll(a);
	}
	
	//
	// 状態
	//

	// 変更されたかを返す
	public boolean isModified()
	{
		return modified;	
	}
	
	// 名前の一覧を返す
	public String[] getNames()
	{
		final String[] names = new String[accounts.size()];

		final Iterator it = accounts.iterator();
		int index = 0;
		while (it.hasNext()) {
			names[index++] = ((AccountInfo)it.next()).name;
		}

		return names;
	}

	// 指定名のアカウント情報を返す
	public AccountInfo getAccountInfoOf(
		String name)	// 取得対象名
	{
		if (name == null) {
			throw new NullPointerException();
		}
		else if (name.length() == 0) {
			throw new IllegalArgumentException();
		}
		
		AccountInfo account = null;
		
		final Iterator it = accounts.iterator();
		int index = 0;
		while (it.hasNext()) {
			AccountInfo info = (AccountInfo)it.next(); 
			if (info.name.equals(name)) {
				account = info;
				break;
			}
		}
		
		return account;
	}

	//
	// 操作
	//
	
	// 追加する
	public void add(
		AccountInfo account)	// 追加アカウント情報
	{
		if (account == null) {
			throw new NullPointerException();
		}
		if (accounts.contains(account)) {
			throw new IllegalArgumentException();
		}
		
		accounts.add(account);
		Collections.sort(accounts, new Comparator() {
			public int compare(Object lhs, Object rhs) {
				return ((AccountInfo)lhs).name.compareToIgnoreCase(((AccountInfo)rhs).name);
			}
		});
		modified = true;
	}

	// 削除する
	public void remove(
		AccountInfo account)	// 削除アカウント情報
	{
		if (account == null) {
			throw new NullPointerException();
		}
		if (accounts.contains(account) == false) {
			throw new IllegalArgumentException();
		}
		
		accounts.remove(account);
		modified = true;
	}

	// ストリームに書きこむ
	void serializeTo(
		OutputStream stream)	// 出力対象ストリーム
		throws IOException
	{
		if (stream == null) {
			throw new NullPointerException();
		}

		final Writer writer = new BufferedWriter(new OutputStreamWriter(stream, CHARSET));
		try {
			final Iterator it = accounts.iterator();
			while (it.hasNext()) {
				((AccountInfo)it.next()).serializeTo(writer);
				writer.write(NEWLINE);
			}
		}
		finally {
			try {
				writer.close();
			}
			catch (Exception ex){
				// Do nothing
				SBKeyHook.reportException(ex);
			}
		}
	}
}
