/*
 * Serializer: シリアライズ
 */
package org.dbc_works.app.sbkeyhook.model;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.OutputStream;

import org.bouncycastle.crypto.InvalidCipherTextException;

import org.dbc_works.lib.io.FileUtilities;
import org.dbc_works.lib.model.AbstractSerializer;
import org.dbc_works.lib.util.Utilities;

import org.dbc_works.app.sbkeyhook.SBKeyHook;

public final class Serializer extends AbstractSerializer
{
	//
	// クラスメソッド(公開)
	//

	// シリアライズ用インスタンスを返す
	public static Serializer getSerializeInstance(
		String password,			// パスワード
		KeyHookDocument document)	// 対象ドキュメント
	{
		if (password == null) {
			throw new NullPointerException();
		}
		else if (password.length() == 0) {
			throw new IllegalArgumentException();
		}
		else if (document == null) {
			throw new NullPointerException();
		}
		
		return new Serializer(password, document);
	}
	
	// デシリアライズ用インスタンスを返す
	public static Serializer getDeserializeInstance(
		String password)	// パスワード
	{
		if (password == null) {
			throw new NullPointerException();
		}
		else if (password.length() == 0) {
			throw new IllegalArgumentException();
		}
		
		return new Serializer(password);
	}
	
	//
	// フィールド(実装)
	//

	private final String password;
	private final KeyHookDocument document;
		
	//
	// コンストラクタ
	//
	private Serializer(
		String pwd,				// パスワード
		KeyHookDocument doc)	// シリアライズ対象ドキュメント
	{
		password = pwd;
		document = doc;
	}
	private Serializer(
		String pwd)	// パスワード
	{
		password = pwd;
		document = null;
	}
	
	//
	// 操作
	//
	
	// デシリアライズする
	public KeyHookDocument deserialize(
		File deserialize)	// デシリアライズ対象ファイル
		throws Exception
	{
		if (document != null) {
			throw new IllegalStateException();
		}
		if (deserialize == null) {
			throw new NullPointerException();
		}
		else if (deserialize.exists() == false) {
			throw new IllegalArgumentException();
		}
		KeyHookDocument document = null;

		final InputStream stream = new FileInputStream(deserialize);
		try {
			ByteArrayOutputStream encryptedStream = new ByteArrayOutputStream();
			FileUtilities.duplicateStream(stream, encryptedStream);			
		
			final byte[] decrypted = new Encryptor(password.getBytes()).decrypt(encryptedStream.toByteArray());
			document = KeyHookDocument.parse(new ByteArrayInputStream(decrypted));
		}
		catch (InvalidCipherTextException ex) {
			Utilities.reportException("", ex);
			document = null;
		}
		finally {
			try {
				stream.close();
			}
			catch (Exception ex) {
				// Do nothing.
				SBKeyHook.reportException(ex);
			}
		}
		
		return document;
	}
	
	//
	// 再定義メソッド(実装)
	//

	// シリアライズする
	protected void doSerialize(
		OutputStream stream)	// 出力対象ストリーム
		throws Exception
	{
		if (document == null) {
			throw new IllegalStateException();
		}
		if (stream == null) {
			throw new NullPointerException();
		}
		
		ByteArrayOutputStream docStream = new ByteArrayOutputStream();
		InputStream encryptedStream = null;
		try {
			document.serializeTo(docStream);
			final byte[] encrypted = new Encryptor(password.getBytes()).encrypt(docStream.toByteArray());
			encryptedStream = new ByteArrayInputStream(encrypted);
			FileUtilities.duplicateStream(encryptedStream, stream);			
		}
		finally {
			if (encryptedStream != null) {
				encryptedStream.close();
			}
			docStream.close();
		}
	}
}
