/*
 * ZaurusJ2MEInfo: J2ME搭載Zaurus環境情報クラス
 */

package org.dbc_works.lib.environment;

import org.dbc_works.lib.io.JavaFileUtilities;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

public final class ZaurusJ2MEInfo extends UnixInfo
{
	/*
	 * 備忘録
	 * Javaがサポートする文字符号化方式は基本エンコーディングセットと拡張エンコーディング
	 * セットのふたつに分かれている。基本エンコーディングセットはいわゆるASCIIとUNICODEを
	 * サポート、拡張エンコーディングセットは加えてさまざまな言語の文字集合用符号化方式を
	 * サポートしている。Zaurusが搭載しているJ2MEはどううやら基本エンコーディングセット
	 * しかサポートしていない模様。なので符号化方式が問題になる場面用に特別な対応をしなければ
	 * ならない。
	 * 参考: http://java.sun.com/j2se/1.3/ja/docs/ja/guide/intl/encoding.doc.html
	 */

	//
	// クラス属性
	//
/*
	// タスクバーなどの情報
	// J2MEではToolkit.getScreenSize()でそれらを除いたサイズが取得できるので不要
 	private static final int titleHeight = 13;
	private static final int taskBarHeight = 19;
	private static final int margin = 2;
*/

	//
	// 構築子
	//
	protected ZaurusJ2MEInfo()
	{
		super();
	}

	//
	// 再定義メソッド(状態)
	//

	// デフォルトの文字符号化方式を返す
	public String getDefaultEncoding()
	{
		return "UTF8";
	}

	// 文字列を指定文字列で分割する
	// 注記; ZaurusのJ2MEにないString#splitメソッド(1.4で追加)の代替メソッド。
	public String[] split(
		String separator,	// 分割文字列
		String target)		// 分割対象文字列
	{
		final List list = new ArrayList();
		String str = target;
		int index = str.indexOf(separator);
		while (0 <= index) {
			list.add(str.substring(0, index));
			str = str.substring(index + separator.length());
			index = str.indexOf(separator);
		}
		list.add(str);
		
		final String[] lines = new String[list.size()];
		int line = 0;
		final Iterator it = list.iterator();
		while (it.hasNext()) {
			lines[line++] = (String)it.next();
		}
		return lines;
	}
	
	// Swingが利用可能かを返す
	public boolean isSwingAvailable()
	{
		return false;
	}

	// ウィンドウタイトルに使える文字が基本ラテンだけかを返す
	public boolean isWindowTitleBasicLatinOnly()
	{
		// CJKなどを設定すると文字化けする
		return true;
	}

	// アイコン設定の要不要を返す
	public boolean isIconRequired()
	{
		return false;
	}

	// フレームのサイズ変更が可能かを返す
	public boolean isFrameResizable()
	{
		// SL-C7x0/8x0ではインプットスタイルとビュースタイルの切り替え時に画面の
		// 表示方向が切り替わる。このときフレームのサイズはシステムによって自動的
		// に調整される(アクティブになったときも同様)。サイズ変更を不可にすると
		// この調整が有効にならないので可にしなければならない。
		// (いずれにせよユーザー操作では変えられない)
		return true;
	}

	// 全画面表示をするかを返す
	public boolean showMainFrameAsFullScreen()
	{
		return true;
	}

	// メインウィンドウ用のサイズを返す
	public Dimension getDefaultMainFrameSize(
		Component component)	// 参照コンポーネント(入力)
	{
		// Zaurusの基本は全画面表示
		return component.getToolkit().getScreenSize();
	}

	// ウィンドウ用のフォントを返す
	public Font getBaseFont(
		Component component)	// 参照コンポーネント(入力)
	{
		// サイズはポイント指定
		// SL-C7x0/C8x0のJ2MEは18ポイントでないと日本語がうまく表示されない、とか。
		return new Font(super.FONTFACE_DEFAULT, Font.PLAIN, 18);
	}

	// メニュー用のフォントを返す
	public Font getMenuFont(
		Component component)	// 参照コンポーネント(入力)
	{
		// 特別なものは指定しない
		return null;
	}

	// マウスの有無を返す
	public boolean hasNoMouse()
	{
		return true;
	}

	//
	// 再定義メソッド(実装)
	//

	// Javaコマンドの名前を返す
	protected String getJavaCmdName()
	{
		return "cvm";
	}

	// Javaコマンドの引数を作成する
	// Zaurus J2MEの引数: -XappName=(name) -cp file:(classpath) (main class name)
	protected String makeJavaCmdArgument(
		String strRunJavaFile,		// 実行ファイル
		Collection colClassPath,	// クラスパス
		String strName)				// 名前
		throws IOException
	{
		File file = new File(strRunJavaFile); 
		if (file.exists() == false) {
			throw new IllegalArgumentException();
		}
	
		JavaFileUtilities.FileType type = JavaFileUtilities.getFileType(file);
		if (type != JavaFileUtilities.FileType.CLASS
		&& type != JavaFileUtilities.FileType.JAR) {
			throw new IllegalArgumentException();
		}

		StringBuffer sb = new StringBuffer();

		if (strName != null && 0 < strName.length()) {
			sb.append("-XappName=");
			sb.append(strName);
			sb.append(' ');
		}
		sb.append("-Duser.language=ja -Duser.region=JP ");

		// クラスパスと実行クラス名を取得する
		// 実行クラス名はJARだったらmanifestから、普通のクラスだったら末尾の"/"から"."までのあいだ
		String strStartClassPath;
		String strStartClassName;
		if (type == JavaFileUtilities.FileType.JAR) {
			strStartClassPath = "file:" + strRunJavaFile;
			strStartClassName = JavaFileUtilities.getJarStartClass(file);
		}
		else {
			String[] astrCmdLineInfo = getCmdLineInfoFromFile(strRunJavaFile);
			strStartClassName = astrCmdLineInfo[0];
			strStartClassPath = astrCmdLineInfo[1];
		}

		// クラスパス引数を追加する
		String strClassPathLine = makeJavaClassPathLine(strStartClassPath, colClassPath); 
		if (0 < strClassPathLine.length()) {
			// SunのEasry Access版では"-Djava.class.path="でなければならないとの情報あり
			sb.append("-cp ");
			sb.append(strClassPathLine);
			sb.append(' ');
		}

		// Javaプログラムを追加する
		sb.append(strStartClassName);

		return sb.toString();
	}
}
