/*
 * LookAndFeelUtilities: 見た目サポートクラス
 * 注記: クラスメソッドのみ定義するのでインスタンスの生成は許さない 
 */
package org.dbc_works.lib.ui.swing;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.LookAndFeel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

public final class LookAndFeelUtilities
{
	//
	// 内部型定義
	//

	// 見た目選択アクションリスナー 
	private static class SelectLafListener implements java.awt.event.ActionListener 
	{
		//
		// 属性
		//

		private final String className;
		private final JFrame target;

		//
		// コンストラクタ
		//
		public SelectLafListener(
			String strClassName,	// クラス名
			JFrame frameTarget)		// 対象フレーム
		{
			className = strClassName;
			target = frameTarget;
		}

		//
		// イベントハンドラ
		//

		// アクション実行
		public void actionPerformed(
			ActionEvent e)	// イベント情報
		{
			try {
				select(className, target);
			}
			catch (UnsupportedLookAndFeelException exp) {
				// Linux版Sun JDK 1.4.2_02ではサポートしない見た目がリストアップされることを
				// 確認した。つまりこの例外は発生しうる
				exp.printStackTrace();
			}
			catch (IllegalAccessException exp) {
				// 発生しないはずの例外
				exp.printStackTrace();
			}
			catch (InstantiationException exp) {
				// 発生しないはずの例外
				exp.printStackTrace();
			}
			catch (ClassNotFoundException exp) {
				// 発生しないはずの例外
				exp.printStackTrace();
			}
		}
	}

	//
	// クラス属性
	//

	private static final boolean DEBUG	= false;

	private static final String PROPKEY_LOOKANDFEEL	= "LookAndFeel";

	//
	// クラスメソッド
	//

	// 見た目変更メニューを作成する
	// 戻り値: 作成したメニュー項目のリスト
	public static List createSelectionMenuItems(
		JFrame frameTarget)			// 変更対象フレーム
	{
		if (DEBUG) {
			if (frameTarget == null) {
				throw new NullPointerException();
			}
		}

		ButtonGroup group = new ButtonGroup();

		UIManager.LookAndFeelInfo lafInfo[] = UIManager.getInstalledLookAndFeels();
		LookAndFeel currentLaf = UIManager.getLookAndFeel();
		List listMenuItem = new ArrayList();

		// 切り替え項目を作成する 
		for (int nIndex = 0; nIndex < lafInfo.length; nIndex++) {
			try {
				// Linux版Sun JDK 1.4.2_02でサポートされない見た目がリストアップされることを
				// 確認したので事前にチェックする 
				LookAndFeel laf = (LookAndFeel)Class.forName(lafInfo[nIndex].getClassName()).newInstance();
				if (laf.isSupportedLookAndFeel()
				&& (JFrame.isDefaultLookAndFeelDecorated() == false
					|| laf.getSupportsWindowDecorations())) {
					JRadioButtonMenuItem menuItem = new JRadioButtonMenuItem(lafInfo[nIndex].getName());
					group.add(menuItem);

					menuItem.addActionListener(new SelectLafListener(lafInfo[nIndex].getClassName(), frameTarget));
					if (currentLaf != null && currentLaf.getName().equals(lafInfo[nIndex].getName())) {
						menuItem.setSelected(true);
					}

					listMenuItem.add(menuItem);
				}
			}
			catch (IllegalAccessException exp) {
				exp.printStackTrace();
			}
			catch (InstantiationException exp) {
				exp.printStackTrace();
			}
			catch (ClassNotFoundException exp) {
				exp.printStackTrace();
			}
		}

		// 修飾項目を作成する

		return listMenuItem;
	}

	// 見た目メニュー項目を選択する
	public static void select(
		String strName,		// メニュー項目の表記
		List lafMenuItems)	// 見た目メニュー項目のリスト 
	{
		if (DEBUG) {
			if (strName == null) {
				throw new NullPointerException();
			}
			else if (strName.length() == 0) {
				throw new IllegalArgumentException();
			}
			if (lafMenuItems == null) {
				throw new NullPointerException();
			}
		}

		final Iterator it = lafMenuItems.iterator();
		while (it.hasNext()) {
			final JRadioButtonMenuItem item = (JRadioButtonMenuItem)it.next();
			if (item.getText().equals(strName)) {
				item.doClick();
				break;
			}
		}
	}

	// 見た目を選択する
	public static void select(
		String strClassName,	// 見た目のクラス名
		JFrame frameTarget)		// 変更対象フレーム
		throws UnsupportedLookAndFeelException,
			IllegalAccessException,
			InstantiationException,
			ClassNotFoundException
	{
		if (DEBUG) {
			if (strClassName == null) {
				throw new NullPointerException();
			}
			else if (strClassName.length() == 0) {
				throw new IllegalArgumentException();
			}
			if (frameTarget == null) {
				throw new NullPointerException();
			}
		}

		UIManager.setLookAndFeel(strClassName);
		SwingUtilities.updateComponentTreeUI(frameTarget);
	}

	// メニュー項目の状態を返す 
	public static Map getMenuItemState(
		List listMenuItem)	// addSelectMenuItemメソッドで作成したメニュー項目のリスト
	{
		if (DEBUG) {
			if (listMenuItem == null) {
				throw new NullPointerException();
			}
		}

		Map map = new HashMap();
		Iterator it = listMenuItem.iterator();
		while (it.hasNext()) {
			JRadioButtonMenuItem item = (JRadioButtonMenuItem)it.next();
			if (item.isSelected()) {
				map.put(PROPKEY_LOOKANDFEEL, item.getText());
				break;
			}
		}
		return map;
	}

	// メニュー項目の状態を設定する 
	public static void setMenuItemState(
		Map mapState,		// 状態のマップ
		List listMenuItem)	// addSelectMenuItemメソッドで作成したメニュー項目のリスト
	{
		if (DEBUG) {
			if (listMenuItem == null) {
				throw new NullPointerException();
			}
		}

		if (mapState.containsKey(PROPKEY_LOOKANDFEEL)) {
			Object laf = mapState.get(PROPKEY_LOOKANDFEEL);
			if (DEBUG) {
				if ((laf instanceof String) == false) {
					throw new IllegalArgumentException();
				}
			}
			select((String)laf, listMenuItem);
		}
	}

	// 子コンポーネントにフォントを設定する
	public static void setFontToComponents(
		java.awt.Font fontSet,			// 設定するフォント
		java.awt.Container container)	// 設定するコンポーネント
	{
		if (DEBUG) {
			if (fontSet == null || container == null) {
				throw new NullPointerException();
			}
		}

		java.awt.Component[] components = container.getComponents();
		for (int nIndex = 0; nIndex < components.length; nIndex++) {
			components[nIndex].setFont(fontSet);
			if (components[nIndex] instanceof java.awt.Container) {
				setFontToComponents(fontSet, (java.awt.Container)components[nIndex]);
			}
		}
	}

	//
	// コンストラクタ
	//
	private LookAndFeelUtilities()
	{
	}
}
